"""victoria.storage.azure_provider

Implementation of a StorageProvider for Azure Blob Storage.

Author:
    Sam Gibson <sgibson@glasswallsolutions.com>
"""
from contextlib import contextmanager
from io import IOBase
import logging
from typing import Generator, ContextManager, Union

from azure.storage.blob.blockblobservice import BlockBlobService

from . import provider


class AzureStorageProvider(provider.StorageProvider):
    """Storage provider for Azure Blob Storage.

    Attributes:
        client (BlockBlobService): The Azure API blob client.
        container (str): The container we are currently using.

    Args:
        account (str): The storage account name to connect to.
        credential (str): The connection string of the storage account.
        container (str): The container within the account to focus on.
    """
    def __init__(self, account: str, credential: str, container: str = None):
        self.client = BlockBlobService(account_name=account,
                                       account_key=credential)
        self.container = container

    def store(self, data: Union[IOBase, str, bytes], key: str) -> None:
        self._ensure_container()
        if issubclass(type(data), IOBase):
            self.client.create_blob_from_stream(self.container, key, data)
        elif type(data) == str:
            self.client.create_blob_from_text(self.container, key, data)
        elif type(data) == bytes:
            self.client.create_blob_from_bytes(self.container, key, data)
        else:
            raise TypeError(f"invalid data type '{type(data)}'")

    def retrieve(self, key: str, stream: IOBase):
        self._ensure_container()
        self.client.get_blob_to_stream(self.container, key, stream)

    def ls(self) -> Generator[str, None, None]:
        self._ensure_container()
        for blob in self.client.list_blobs(self.container):
            yield blob.name

    def _ensure_container(self):
        """Ensure that we are actually connected to a container."""
        if not self.container:
            raise ValueError("storage container has not been set")


"""setup.py

Used for installing Victoria via pip.
"""

from setuptools import setup, find_packages


def repo_file_as_string(file_path: str) -> str:
    with open(file_path, "r") as repo_file:
        return repo_file.read()


setup(dependency_links=[],
      install_requires=[
          "appdirs>=1.4.3", "click>=7.0", "marshmallow>=3.2.1",
          "pyyaml>=5.1.2", "azure-storage>=0.36.0", "cryptography>=2.8",
          "azure-keyvault>=4.0.0", "azure-identity>=1.3.0", "dpath>=2.0.1"
      ],
      name="victoria",
      version="v0.4.0",
      description="Automation toolbelt",
      long_description=repo_file_as_string("README.md"),
      long_description_content_type="text/markdown",
      author="Sam Gibson",
      author_email="sgibson@glasswallsolutions.com",
      url="https://github.com/glasswall-sre/victoria",
      classifiers=[
          "Development Status :: 4 - Beta", "Environment :: Console",
          "Intended Audience :: Developers",
          "Intended Audience :: System Administrators",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Programming Language :: Python :: 3.8", "Topic :: Terminals",
          "Topic :: Utilities"
      ],
      keywords="cli automation sre plugin",
      packages=find_packages(".") +
      find_packages("example_plugins/victoria_config") +
      find_packages("example_plugins/victoria_store") +
      find_packages("example_plugins/victoria_encrypt"),
      package_dir={
          "victoria_config": "example_plugins/victoria_config/victoria_config",
          "victoria_store": "example_plugins/victoria_store/victoria_store",
          "victoria_encrypt":
          "example_plugins/victoria_encrypt/victoria_encrypt"
      },
      entry_points="""
        [console_scripts]
        victoria=victoria.script.victoria:main
    """,
      python_requires=">=3.6",
      include_package_data=True,
      package_data={"victoria": ["victoria_example.yaml"]})
